@@ -1,22 +1,18 @@
 """This module creates the database."""
 import boto3
-from therapy.schemas import NamespacePrefix, SourceName
+from therapy import PREFIX_LOOKUP
 from botocore.exceptions import ClientError
 from boto3.dynamodb.conditions import Key
 from os import environ
 from typing import Optional, Dict, List, Any
 import logging
+import click
+import sys
 
 logger = logging.getLogger('therapy')
 logger.setLevel(logging.DEBUG)
 
 
-# should this be located in a shared module
-PREFIX_LOOKUP = {v.value: SourceName[k].value
-                 for k, v in NamespacePrefix.__members__.items()
-                 if k in SourceName.__members__.keys()}
-
-
 class Database:
     """The database class."""
 
@@ -26,25 +22,40 @@ class Database:
         :param str db_url: database endpoint URL to connect to
         :param str region_name: AWS region name to use
         """
-        if db_url:
-            boto_params = {
-                'region_name': region_name,
-                'endpoint_url': db_url
-            }
-        elif 'THERAPY_NORM_DB_URL' in environ.keys():
-            boto_params = {
-                'region_name': region_name,
-                'endpoint_url': environ['THERAPY_NORM_DB_URL']
-            }
-        else:
+        if 'THERAPY_NORM_PROD' in environ or 'THERAPY_NORM_EB_PROD' in environ:
             boto_params = {
                 'region_name': region_name
             }
+            if 'THERAPY_NORM_EB_PROD' not in environ:
+                # EB Instance should not have to confirm.
+                # This is used only for using production via CLI
+                if click.confirm("Are you sure you want to use the "
+                                 "production database?", default=False):
+                    click.echo("***THERAPY PRODUCTION DATABASE IN USE***")
+                else:
+                    click.echo("Exiting.")
+                    sys.exit()
+        else:
+            if db_url:
+                endpoint_url = db_url
+            elif 'THERAPY_NORM_DB_URL' in environ:
+                endpoint_url = environ['THERAPY_NORM_DB_URL']
+            else:
+                endpoint_url = 'http://localhost:8000'
+            click.echo(f"***Using Therapy Database Endpoint: "
+                       f"{endpoint_url}***")
+
+            boto_params = {
+                'region_name': region_name,
+                'endpoint_url': endpoint_url
+            }
+
         self.dynamodb = boto3.resource('dynamodb', **boto_params)
         self.dynamodb_client = boto3.client('dynamodb', **boto_params)
 
-        # create tables if nonexistant if not connecting to remote database
-        if db_url or 'THERAPY_NORM_DB_URL' in environ.keys():
+        # Table creation for local database
+        if 'THERAPY_NORM_PROD' not in environ and \
+                'THERAPY_NORM_EB_PROD' not in environ:
             existing_tables = self.dynamodb_client.list_tables()['TableNames']
             self.create_therapies_table(existing_tables)
             self.create_meta_data_table(existing_tables)
@@ -85,8 +96,11 @@ class Database:
                     {
                         'AttributeName': 'src_name',
                         'AttributeType': 'S'
+                    },
+                    {
+                        'AttributeName': 'item_type',
+                        'AttributeType': 'S'
                     }
-
                 ],
                 GlobalSecondaryIndexes=[
                     {
@@ -104,6 +118,22 @@ class Database:
                             'ReadCapacityUnits': 10,
                             'WriteCapacityUnits': 10
                         }
+                    },
+                    {
+                        'IndexName': 'item_type_index',
+                        'KeySchema': [
+                            {
+                                'AttributeName': 'item_type',
+                                'KeyType': 'HASH'
+                            }
+                        ],
+                        'Projection': {
+                            'ProjectionType': 'KEYS_ONLY'
+                        },
+                        'ProvisionedThroughput': {
+                            'ReadCapacityUnits': 10,
+                            'WriteCapacityUnits': 10
+                        }
                     }
                 ],
                 ProvisionedThroughput={
@@ -167,7 +197,9 @@ class Database:
                 response = self.therapies.query(KeyConditionExpression=exp)
                 return response['Items'][0]
         except ClientError as e:
-            logger.error(e.response['Error']['Message'])
+            logger.error(f"boto3 client error on get_records_by_id for "
+                         f"search term {concept_id}: "
+                         f"{e.response['Error']['Message']}")
             return None
         except KeyError:  # record doesn't exist
             return None
@@ -190,50 +222,57 @@ class Database:
             matches = self.therapies.query(KeyConditionExpression=filter_exp)
             return matches.get('Items', None)
         except ClientError as e:
-            logger.error(e.response['Error']['Message'])
+            logger.error(f"boto3 client error on get_records_by_type for "
+                         f"search term {query}: "
+                         f"{e.response['Error']['Message']}")
             return []
 
-    def get_merged_record(self, merge_ref) -> Optional[Dict]:
-        """Fetch merged record from given reference.
-
-        :param str merge_ref: key for merged record, formated as a string
-            of grouped concept IDs separated by vertical bars, ending with
-            `##merger`. Must be correctly-cased.
-        :return: complete merged record if lookup successful, None otherwise
-        """
-        try:
-            match = self.therapies.get_item(Key={
-                'label_and_type': merge_ref.lower(),
-                'concept_id': merge_ref
-            })
-            return match['Item']
-        except ClientError as e:
-            logger.error(e.response['Error']['Message'])
-            return None
-        except KeyError:
-            return None
-
-    def add_record(self, record: Dict, record_type: str):
+    def add_record(self, record: Dict, record_type="identity"):
         """Add new record to database.
 
-        :param Dict record: record (of any type) to upload. Must include
-            `concept_id` key. If record is of the `identity` type, the
-            concept_id must be correctly-cased.
-        :param str record_type: one of 'identity', 'label', 'alias',
-            'trade_name', or 'merger'.
+        :param Dict record: record to upload
+        :param str record_type: type of record (either 'identity' or 'merger')
         """
-        if record_type in ('label', 'trade_name', 'alias', 'identity'):
-            prefix = record['concept_id'].split(':')[0]
-            record['src_name'] = PREFIX_LOOKUP[prefix]
-        record['label_and_type'] = f'{record["concept_id"].lower()}##{record_type}'  # noqa: E501
-        self.batch.put_item(Item=record)
+        id_prefix = record['concept_id'].split(':')[0].lower()
+        record['src_name'] = PREFIX_LOOKUP[id_prefix]
+        label_and_type = f'{record["concept_id"].lower()}##{record_type}'
+        record['label_and_type'] = label_and_type
+        record['item_type'] = record_type
+        try:
+            self.batch.put_item(Item=record)
+        except ClientError as e:
+            logger.error("boto3 client error on add_record for "
+                         f"{record['concept_id']}: "
+                         f"{e.response['Error']['Message']}")
+
+    def add_ref_record(self, term: str, concept_id: str, ref_type: str):
+        """Add auxiliary/reference record to database.
+
+        :param str term: referent term
+        :param str concept_id: concept ID to refer to
+        :param str ref_type: one of ('alias', 'label', 'other_id', 'xref')
+        """
+        label_and_type = f'{term.lower()}##{ref_type}'
+        src_name = PREFIX_LOOKUP[concept_id.split(':')[0].lower()]
+        record = {
+            'label_and_type': label_and_type,
+            'concept_id': concept_id.lower(),
+            'src_name': src_name,
+            'item_type': ref_type,
+        }
+        try:
+            self.batch.put_item(Item=record)
+        except ClientError as e:
+            logger.error(f"boto3 client error adding reference {term} for "
+                         f"{concept_id} with match type {ref_type}: "
+                         f"{e.response['Error']['Message']}")
 
     def update_record(self, concept_id: str, field: str, new_value: Any):
         """Update the field of an individual record to a new value.
 
         :param str concept_id: record to update
         :param str field: name of field to update
-        :parm str new_value: new value
+        :param Any new_value: new value
         """
         key = {
             'label_and_type': f'{concept_id.lower()}##identity',
@@ -246,4 +285,10 @@ class Database:
                                        UpdateExpression=update_expression,
                                        ExpressionAttributeValues=update_values)
         except ClientError as e:
-            logger.error(e.response['Error']['Message'])
+            logger.error(f"boto3 client error in `database.update_record()`: "
+                         f"{e.response['Error']['Message']}")
+
+    def flush_batch(self):
+        """Flush internal batch_writer."""
+        self.batch.__exit__(*sys.exc_info())
+        self.batch = self.therapies.batch_writer()
