@@ -6,9 +6,10 @@ Library of Medicine (NLM), National Institutes of Health, Department of Health
  endorse or recommend this or any other product."
 """
 from .base import Base
-from therapy import PROJECT_ROOT, DownloadException
+from therapy import PROJECT_ROOT, DownloadException, OTHER_IDENTIFIERS, XREFS
+import therapy
 from therapy.database import Database
-from therapy.schemas import SourceName, NamespacePrefix, Meta, Drug, \
+from therapy.schemas import SourceName, NamespacePrefix, SourceMeta, Drug, \
     ApprovalStatus
 import csv
 import datetime
@@ -21,6 +22,7 @@ import shutil
 import zipfile
 import re
 import yaml
+from pathlib import Path
 
 logger = logging.getLogger('therapy')
 logger.setLevel(logging.DEBUG)
@@ -36,9 +38,9 @@ ALIASES = ['SYN', 'SY', 'TMSY', 'PM',
 # Semantic branded drug
 TRADE_NAMES = ['BD', 'BN', 'SBD']
 
-# Allowed xrefs that have Source Level Restriction 0 or 1
-XREFS = ['ATC', 'CVX', 'DRUGBANK', 'MMSL', 'MSH', 'MTHCMSFRF', 'MTHSPL',
-         'RXNORM', 'USP', 'VANDF']
+# Allowed rxnorm xrefs that have Source Level Restriction 0 or 1
+RXNORM_XREFS = ['ATC', 'CVX', 'DRUGBANK', 'MMSL', 'MSH', 'MTHCMSFRF', 'MTHSPL',
+                'RXNORM', 'USP', 'VANDF']
 
 
 class RxNorm(Base):
@@ -53,8 +55,6 @@ class RxNorm(Base):
         :param Database database: Access to DynamoDB.
         :param str data_url: URL to RxNorm data files.
         """
-        self._other_id_srcs = {src for src in SourceName.__members__}
-        self._xref_srcs = {src for src in NamespacePrefix.__members__}
         self._data_url = data_url
         self.database = database
         self._added_ids = []
@@ -112,25 +112,27 @@ class RxNorm(Base):
             uri = 'https://download.nlm.nih.gov/umls/kss/' \
                   'rxnorm/RxNorm_full_current.zip'
 
+            rxnorm_path = str(Path(therapy.__file__).resolve().parents[0] / 'data' / 'rxnorm' / 'RxNorm_full_current.zip')  # noqa: E501
+            environ['RXNORM_PATH'] = rxnorm_path
+
             # Source:
             # https://documentation.uts.nlm.nih.gov/automating-downloads.html
-            subprocess.call(['bash', f'{PROJECT_ROOT}/therapy/etl/'
+            subprocess.call(['bash', f'{PROJECT_ROOT}/etl/'
                                      f'rxnorm_download.sh', uri])
 
-            zip_path = rxn_dir / 'RxNorm_full_current.zip'
-            shutil.move(PROJECT_ROOT / 'RxNorm_full_current.zip', zip_path)
-
-            with zipfile.ZipFile(zip_path, 'r') as zf:
+            with zipfile.ZipFile(rxnorm_path, 'r') as zf:
                 zf.extractall(rxn_dir)
 
-            remove(zip_path)
+            remove(rxnorm_path)
             shutil.rmtree(rxn_dir / 'prescribe')
             shutil.rmtree(rxn_dir / 'scripts')
 
-            # get version
             readme = sorted([fn for fn in rxn_dir.iterdir() if fn.name.
                             startswith('Readme')])[0]
-            version = str(readme).split('.')[0].split('_')[-1]
+
+            # get version
+            version = str(readme).split('/')[-1].split('.')[0].split('_')[-1]
+
             self._version = datetime.datetime.strptime(
                 version, '%m%d%Y').strftime('%Y%m%d')
             remove(readme)
@@ -175,7 +177,7 @@ class RxNorm(Base):
             sbdfs = dict()  # Link ingredient to brand
             brands = dict()  # Get RXNORM|BN to concept_id
             for row in rff_data:
-                if row[11] in XREFS:
+                if row[11] in RXNORM_XREFS:
                     concept_id = f"{NamespacePrefix.RXNORM.value}:{row[0]}"
                     if row[12] == 'BN' and row[11] == 'RXNORM':
                         brands[row[14]] = concept_id
@@ -219,7 +221,8 @@ class RxNorm(Base):
                                                         in value else [],
                             xrefs=value['xrefs'] if 'xrefs' in value else [],
                             trade_names=value['trade_names'] if 'trade_names'
-                                                                in value else []  # noqa: E501
+                                                                in value
+                                                                else []
                         )
                         self._load_therapy(params, batch)
 
@@ -254,6 +257,7 @@ class RxNorm(Base):
         self._load_label_types(params, batch)
         params['src_name'] = SourceName.RXNORM.value
         params['label_and_type'] = f"{params['concept_id'].lower()}##identity"
+        params['item_type'] = 'identity'
         try:
             batch.put_item(Item=params)
             self._added_ids.append(params['concept_id'])
@@ -279,6 +283,11 @@ class RxNorm(Base):
             self._load_label_type(params, batch, 'trade_name', 'trade_names')
         if 'aliases' in params:
             self._load_label_type(params, batch, 'alias', 'aliases')
+        if 'other_identifiers' in params:
+            self._load_label_type(params, batch, 'other_id',
+                                  'other_identifiers')
+        if 'xrefs' in params:
+            self._load_label_type(params, batch, 'xref', 'xrefs')
 
     def _load_label_type(self, params, batch, label_type_sing, label_type_pl):
         """Insert alias, trade_name, or label data into the database.
@@ -296,7 +305,8 @@ class RxNorm(Base):
             t = {
                 'label_and_type': f"{t.lower()}##{label_type_sing}",
                 'concept_id': f"{params['concept_id'].lower()}",
-                'src_name': SourceName.RXNORM.value
+                'src_name': SourceName.RXNORM.value,
+                'item_type': label_type_sing
             }
             try:
                 batch.put_item(Item=t)
@@ -377,7 +387,8 @@ class RxNorm(Base):
                         'label_and_type':
                             f"{brands.get(tn)}##rx_brand",
                         'concept_id': value['concept_id'],
-                        'src_name': SourceName.RXNORM.value
+                        'src_name': SourceName.RXNORM.value,
+                        'item_type': 'rx_brand'
                     })
 
     def _add_str_field(self, params, row, precise_ingredient, drug_forms,
@@ -442,13 +453,13 @@ class RxNorm(Base):
         """
         if row[11]:
             other_id_xref = row[11].upper()
-            if other_id_xref in self._other_id_srcs:
+            if other_id_xref in OTHER_IDENTIFIERS:
                 source_id =\
                     f"{NamespacePrefix[other_id_xref].value}:{row[13]}"
                 if source_id != params['concept_id']:
                     # Sometimes concept_id is included in the source field
                     self._add_term(params, source_id, 'other_identifiers')
-            elif other_id_xref in self._xref_srcs:
+            elif other_id_xref in XREFS:
                 source_id = f"{NamespacePrefix[other_id_xref].value}:{row[13]}"
                 self._add_term(params, source_id, 'xrefs')
             else:
@@ -456,17 +467,18 @@ class RxNorm(Base):
 
     def _load_meta(self):
         """Add RxNorm metadata."""
-        meta = Meta(data_license='UMLS Metathesaurus',
-                    data_license_url='https://www.nlm.nih.gov/research/umls/'
-                                     'rxnorm/docs/termsofservice.html',
-                    version=self._version,
-                    data_url=self._data_url,
-                    rdp_url=None,
-                    data_license_attributes={
-                        'non_commercial': False,
-                        'share_alike': False,
-                        'attribution': True
-                    })
+        meta = SourceMeta(
+            data_license='UMLS Metathesaurus',
+            data_license_url='https://www.nlm.nih.gov/research/umls/rxnorm/docs/termsofservice.html',  # noqa: E501
+            version=self._version,
+            data_url=self._data_url,
+            rdp_url=None,
+            data_license_attributes={
+                'non_commercial': False,
+                'share_alike': False,
+                'attribution': True
+            }
+        )
         params = dict(meta)
         params['src_name'] = SourceName.RXNORM.value
         self.database.metadata.put_item(Item=params)
