@@ -1,9 +1,8 @@
 """This module defines the Wikidata ETL methods."""
-from .base import Base, IDENTIFIER_PREFIXES
+from .base import Base
 from therapy import PROJECT_ROOT
 from therapy.schemas import SourceName, NamespacePrefix, \
-    SourceIDAfterNamespace, Meta
-from therapy.database import Database
+    SourceIDAfterNamespace, SourceMeta
 import json
 import logging
 from typing import Dict, List
@@ -14,6 +13,19 @@ import datetime
 logger = logging.getLogger('therapy')
 logger.setLevel(logging.DEBUG)
 
+# Prefixes for translating ID namespaces
+IDENTIFIER_PREFIXES = {
+    'casRegistry': NamespacePrefix.CASREGISTRY.value,
+    'ChemIDplus': NamespacePrefix.CHEMIDPLUS.value,
+    'pubchemCompound': NamespacePrefix.PUBCHEMCOMPOUND.value,
+    'pubchemSubstance': NamespacePrefix.PUBCHEMSUBSTANCE.value,
+    'chembl': NamespacePrefix.CHEMBL.value,
+    'rxnorm': NamespacePrefix.RXNORM.value,
+    'drugbank': NamespacePrefix.DRUGBANK.value,
+    'wikidata': NamespacePrefix.WIKIDATA.value,
+}
+
+
 SPARQL_QUERY = """
     SELECT ?item ?itemLabel ?casRegistry ?pubchemCompound
            ?pubchemSubstance ?chembl
@@ -52,7 +64,7 @@ class Wikidata(Base):
     """Extract, transform, and load the Wikidata source into therapy.db."""
 
     def __init__(self,
-                 database: Database,
+                 database,
                  data_path: Path = PROJECT_ROOT / 'data' / 'wikidata'):
         """Initialize wikidata ETL class.
 
@@ -97,28 +109,28 @@ class Wikidata(Base):
 
     def _load_meta(self):
         """Add Wikidata metadata."""
-        metadata = Meta(src_name=SourceName.WIKIDATA.value,
-                        data_license='CC0 1.0',
-                        data_license_url='https://creativecommons.org/publicdomain/zero/1.0/',  # noqa: E501
-                        version=self._version,
-                        data_url=None,
-                        rdp_url=None,
-                        data_license_attributes={
-                            'non_commercial': False,
-                            'share_alike': False,
-                            'attribution': False
-                        })
+        metadata = SourceMeta(src_name=SourceName.WIKIDATA.value,
+                              data_license='CC0 1.0',
+                              data_license_url='https://creativecommons.org/publicdomain/zero/1.0/',  # noqa: E501
+                              version=self._version,
+                              data_url=None,
+                              rdp_url=None,
+                              data_license_attributes={
+                                  'non_commercial': False,
+                                  'share_alike': False,
+                                  'attribution': False
+                              })
         params = dict(metadata)
         params['src_name'] = SourceName.WIKIDATA.value
         self.database.metadata.put_item(Item=params)
 
     def _transform_data(self):
         """Transform the Wikidata source data."""
+        from therapy import OTHER_IDENTIFIERS
         with open(self._data_src, 'r') as f:
             records = json.load(f)
 
             items = dict()
-            normalizer_srcs = {src for src in SourceName.__members__}
 
             for record in records:
                 record_id = record['item'].split('/')[-1]
@@ -126,6 +138,7 @@ class Wikidata(Base):
                 if concept_id not in items.keys():
                     item = dict()
                     item['label_and_type'] = f"{concept_id.lower()}##identity"
+                    item['item_type'] = 'identity'
                     item['concept_id'] = concept_id
                     item['src_name'] = SourceName.WIKIDATA.value
 
@@ -138,7 +151,7 @@ class Wikidata(Base):
                             if key.upper() == 'CASREGISTRY':
                                 key = SourceName.CHEMIDPLUS.value
 
-                            if key.upper() in normalizer_srcs:
+                            if key.upper() in OTHER_IDENTIFIERS:
                                 if key != 'chembl':
                                     fmted_other_id = \
                                         f"{IDENTIFIER_PREFIXES[key]}:" \
@@ -177,27 +190,39 @@ class Wikidata(Base):
         if 'aliases' in item:
             item['aliases'] = list(set(item['aliases']))
 
-            if len({a.casefold() for a in item['aliases']}) > 20:  # noqa: E501
+            if len({a.casefold() for a in item['aliases']}) > 20:
                 del item['aliases']
 
-        batch.put_item(Item=item)
-        self._added_ids.append(item['concept_id'])
         concept_id_lower = item['concept_id'].lower()
 
-        if 'aliases' in item.keys():
-            aliases = {alias.lower() for alias in item['aliases']}
-            for alias in aliases:
-                pk = f"{alias}##alias"
-                batch.put_item(Item={
-                    'label_and_type': pk,
-                    'concept_id': concept_id_lower,
-                    'src_name': SourceName.WIKIDATA.value
-                })
-
         if 'label' in item.keys():
             pk = f"{item['label'].lower()}##label"
             batch.put_item(Item={
                 'label_and_type': pk,
                 'concept_id': concept_id_lower,
-                'src_name': SourceName.WIKIDATA.value
+                'src_name': SourceName.WIKIDATA.value,
+                'item_type': 'label'
             })
+
+        for field_type, field in (('alias', 'aliases'),
+                                  ('other_id', 'other_identifiers'),
+                                  ('xref', 'xrefs')):
+            values = item.get(field)
+            if values:
+                keys = {value.casefold() for value in values}
+                if field == 'aliases' and len(keys) > 20:
+                    del item['aliases']
+                    continue
+                for key in keys:
+                    pk = f"{key}##{field_type}"
+                    batch.put_item(Item={
+                        'label_and_type': pk,
+                        'concept_id': concept_id_lower,
+                        'src_name': SourceName.NCIT.value,
+                        'item_type': field_type
+                    })
+            else:
+                del item[field]
+
+        batch.put_item(Item=item)
+        self._added_ids.append(item['concept_id'])
