@@ -4,6 +4,7 @@ therapy records.
 from typing import List, Optional, Dict, Union, Any, Type
 from pydantic import BaseModel, StrictBool
 from enum import Enum, IntEnum
+from datetime import datetime
 
 
 class Therapy(BaseModel):
@@ -108,10 +109,21 @@ class MatchType(IntEnum):
     LABEL = 80
     TRADE_NAME = 80
     ALIAS = 60
+    OTHER_ID = 60
+    XREF = 60
     FUZZY_MATCH = 20
     NO_MATCH = 0
 
 
+class SourcePriority(IntEnum):
+    """Define constraints for Source Priority Rankings."""
+
+    RXNORM = 1
+    NCIT = 2
+    CHEMIDPLUS = 5
+    WIKIDATA = 6
+
+
 class SourceName(str, Enum):
     """Define string constraints to ensure consistent capitalization."""
 
@@ -134,6 +146,15 @@ class SourceIDAfterNamespace(Enum):
     RXNORM = ""
 
 
+class ProhibitedSources(Enum):
+    """Define constraints for sources that are prohibited in normalize
+    endpoint.
+    """
+
+    CHEMBL = SourceName.CHEMBL.value
+    DRUGBANK = SourceName.DRUGBANK.value
+
+
 class NamespacePrefix(Enum):
     """Define string constraints for namespace prefixes on concept IDs."""
 
@@ -178,7 +199,7 @@ class DataLicenseAttributes(BaseModel):
     attribution: StrictBool
 
 
-class Meta(BaseModel):
+class SourceMeta(BaseModel):
     """Metadata for a given source to return in response object."""
 
     data_license: str
@@ -193,7 +214,7 @@ class Meta(BaseModel):
 
         @staticmethod
         def schema_extra(schema: Dict[str, Any],
-                         model: Type['Meta']) -> None:
+                         model: Type['SourceMeta']) -> None:
             """Configure OpenAPI schema"""
             if 'title' in schema.keys():
                 schema.pop('title', None)
@@ -222,7 +243,7 @@ class MatchesKeyed(BaseModel):
 
     match_type: MatchType
     records: List[Drug]
-    meta_: Meta
+    source_meta_: SourceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -238,7 +259,7 @@ class MatchesKeyed(BaseModel):
             schema['example'] = {
                 'match_type': 0,
                 'records': [],
-                'meta_': {
+                'source_meta_': {
                     'data_license': 'CC BY-SA 3.0',
                     'data_license_url':
                         'https://creativecommons.org/licenses/by-sa/3.0/',
@@ -263,7 +284,7 @@ class MatchesListed(BaseModel):
     source: SourceName
     match_type: MatchType
     records: List[Drug]
-    meta_: Meta
+    source_meta_: SourceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -280,7 +301,7 @@ class MatchesListed(BaseModel):
                 'normalizer': 'ChEMBL',
                 'match_type': 0,
                 'records': [],
-                'meta_': {
+                'source_meta_': {
                     'data_license': 'CC BY-SA 3.0',
                     'data_license_url':
                         'https://creativecommons.org/licenses/by-sa/3.0/',
@@ -297,128 +318,64 @@ class MatchesListed(BaseModel):
             }
 
 
-class MergedMatch(BaseModel):
-    """Represent merged concept in response to client.
+class Extension(BaseModel):
+    """Value Object Descriptor Extension class."""
 
-    Currently, does not include ApprovalStatus data because no included
-    sources provide it.
+    type: str
+    name: str
+    value: Union[bool, List[str]]
+
+
+class ValueObjectDescriptor(BaseModel):
+    """VOD class. Provide additional Extension classes for trade_names and
+    references to non-normalized sources.
     """
 
-    label: Optional[str]
-    concept_ids: List[str]
-    aliases: Optional[List[str]]
-    trade_names: Optional[List[str]]
+    id: str
+    type: str
+    value: Any
+    label: str
     xrefs: Optional[List[str]]
+    alternate_labels: Optional[List[str]]
+    extensions: Optional[List[Extension]]
+
+
+class ServiceMeta(BaseModel):
+    """Metadata regarding the therapy-normalization service."""
+
+    name = "thera-py"
+    version: str
+    response_datetime: datetime
+    url: str
 
     class Config:
         """Enables orm_mode"""
 
         @staticmethod
         def schema_extra(schema: Dict[str, Any],
-                         model: Type['MergedMatch']) -> None:
+                         model: Type['SourceMeta']) -> None:
             """Configure OpenAPI schema"""
             if 'title' in schema.keys():
                 schema.pop('title', None)
             for prop in schema.get('properties', {}).values():
                 prop.pop('title', None)
             schema['example'] = {
-                "label": "cisplatin",
-                "concept_ids": [
-                    "rxcui:2555",
-                    "ncit:C376",
-                    "chemidplus:15663-27-1",
-                    "wikidata:Q412415",
-                    "wikidata:Q47522001"
-                ],
-                "aliases": [
-                    "cis Platinum",
-                    "cis-Diaminedichloroplatinum",
-                    "cis-Dichlorodiammineplatinum(II)",
-                    "CDDP",
-                    "Platinol",
-                    "cis-Diamminedichloroplatinum",
-                    "Platinol-AQ",
-                    "cis-Diamminedichloroplatinum(II)",
-                    "CISplatin",
-                    "Diamminodichloride, Platinum",
-                    "cis Diamminedichloroplatinum",
-                    "Platinum Diamminodichloride",
-                    "cis-diamminedichloroplatinum(II)",
-                    "DDP",
-                    "CIS-DDP",
-                    "1,2-Diaminocyclohexaneplatinum II citrate",
-                    "Cis-DDP",
-                    "Dichlorodiammineplatinum",
-                    "cis-Platinum"
-                ],
-                "trade_names": [
-                    "Platinol",
-                    "Cisplatin"
-                ],
-                "xrefs": [
-                    "pubchem.compound:5702198",
-                    "atc:L01XA01",
-                    "fda:Q20Q21Q62J",
-                    "mesh:D002945",
-                    "usp:m17910",
-                    "mthspl:Q20Q21Q62J",
-                    "vandf:4018139",
-                    "umls:C0008838",
-                    "mmsl:d00195",
-                    "mmsl:31747",
-                    "mmsl:4456"
-                ]
-            }
-
-
-class Service(BaseModel):
-    """Core response schema containing matches for each source"""
-
-    query: str
-    warnings: Optional[Dict]
-    source_matches: Union[Dict[SourceName, MatchesKeyed], List[MatchesListed]]
-
-    class Config:
-        """Enables orm_mode"""
-
-        @staticmethod
-        def schema_extra(schema: Dict[str, Any],
-                         model: Type['Service']) -> None:
-            """Configure OpenAPI schema"""
-            if 'title' in schema.keys():
-                schema.pop('title', None)
-            for prop in schema.get('properties', {}).values():
-                prop.pop('title', None)
-            schema['example'] = {
-                'query': 'CISPLATIN',
-                'warnings': None,
-                'meta_': {
-                    'data_license': 'CC BY-SA 3.0',
-                    'data_license_url':
-                        'https://creativecommons.org/licenses/by-sa/3.0/',
-                    'version': '27',
-                    'data_url':
-                        'http://ftp.ebi.ac.uk/pub/databases/chembl/ChEMBLdb/releases/chembl_27/',  # noqa: E501
-                    'rdp_url': 'http://reusabledata.org/chembl.html',
-                    'data_license_attributes': {
-                        'non_commercial': False,
-                        'share_alike': True,
-                        'attribution': True
-                    }
-                }
+                'name': 'thera-py',
+                'version': '0.1.0',
+                'response_datetime': '2021-04-05T16:44:15.367831',
+                'url': 'https://github.com/cancervariants/therapy-normalization'  # noqa: E501
             }
 
 
 class NormalizationService(BaseModel):
-    """Response containing one or more merged records and relevant source
-    data.
-    """
+    """Response containing one or more merged records and source data."""
 
     query: str
     warnings: Optional[Dict]
     match_type: MatchType
-    record: Optional[MergedMatch]
-    meta_: Optional[Dict[SourceName, Meta]]
+    value_object_descriptor: Optional[ValueObjectDescriptor]
+    source_meta_: Optional[Dict[SourceName, SourceMeta]]
+    service_meta_: ServiceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -435,55 +392,65 @@ class NormalizationService(BaseModel):
                 "query": "cisplatin",
                 "warnings": None,
                 "match_type": 80,
-                "record": {
+                "value_object_descriptor": {
+                    "id": "normalize.therapy:cisplatin",
+                    "type": "TherapyDescriptor",
+                    "value": {
+                        "type": "Therapy",
+                        "id": "rxcui:2555"
+                    },
                     "label": "cisplatin",
-                    "concept_ids": [
-                        "rxcui:2555",
-                        "ncit:C376",
-                        "chemidplus:15663-27-1",
-                        "wikidata:Q412415",
-                        "wikidata:Q47522001"
-                    ],
-                    "aliases": [
-                        "cis Platinum",
-                        "cis-Diaminedichloroplatinum",
-                        "cis-Dichlorodiammineplatinum(II)",
-                        "CDDP",
-                        "Platinol",
-                        "cis-Diamminedichloroplatinum",
-                        "Platinol-AQ",
-                        "cis-Diamminedichloroplatinum(II)",
-                        "CISplatin",
-                        "Diamminodichloride, Platinum",
-                        "cis Diamminedichloroplatinum",
-                        "Platinum Diamminodichloride",
-                        "cis-diamminedichloroplatinum(II)",
-                        "DDP",
-                        "CIS-DDP",
-                        "1,2-Diaminocyclohexaneplatinum II citrate",
-                        "Cis-DDP",
-                        "Dichlorodiammineplatinum",
-                        "cis-Platinum"
-                    ],
-                    "trade_names": [
-                        "Platinol",
-                        "Cisplatin"
-                    ],
                     "xrefs": [
-                        "pubchem.compound:5702198",
-                        "atc:L01XA01",
-                        "fda:Q20Q21Q62J",
-                        "mesh:D002945",
-                        "usp:m17910",
-                        "mthspl:Q20Q21Q62J",
-                        "vandf:4018139",
-                        "umls:C0008838",
-                        "mmsl:d00195",
-                        "mmsl:31747",
-                        "mmsl:4456"
+                        "ncit:C376", "chemidplus:15663-27-1",
+                        "wikidata:Q412415"
+                    ],
+                    "alternate_labels": [
+                        "CIS-DDP", "cis Platinum", "DDP",
+                        "Dichlorodiammineplatinum",
+                        "1,2-Diaminocyclohexaneplatinum II citrate",
+                        "CISplatin",
+                        "cis Diamminedichloroplatinum",
+                        "CDDP",
+                        "Diamminodichloride, Platinum",
+                        "cis-Dichlorodiammineplatinum(II)",
+                        "cis-Platinum",
+                        "cis-diamminedichloroplatinum(II)",
+                        "cis-Diamminedichloroplatinum(II)",
+                        "Cis-DDP",
+                        "cis-Diamminedichloroplatinum",
+                        "cis-Diaminedichloroplatinum",
+                        "Platinol-AQ", "Platinol",
+                        "Platinum Diamminodichloride"
+                    ],
+                    "extensions": [
+                        {
+                            "type": "Extension",
+                            "name": "trade_names",
+                            "value": [
+                                "Platinol", "Cisplatin"
+                            ]
+                        },
+                        {
+                            "type": "Extension",
+                            "name": "associated_with",
+                            "value": [
+                                "atc:L01XA01",
+                                "mmsl:4456",
+                                "chebi:CHEBI:27899",
+                                "pubchem.compound:5702198",
+                                "umls:C0008838",
+                                "usp:m17910",
+                                "fda:Q20Q21Q62J",
+                                "mmsl:d00195",
+                                "mthspl:Q20Q21Q62J",
+                                "mmsl:31747",
+                                "mesh:D002945",
+                                "vandf:4018139"
+                            ]
+                        }
                     ]
                 },
-                "meta_": {
+                "source_meta_": {
                     "RxNorm": {
                         "data_license": "UMLS Metathesaurus",
                         "data_license_url": "https://www.nlm.nih.gov/research/umls/rxnorm/docs/termsofservice.html",  # noqa: E501
@@ -511,8 +478,7 @@ class NormalizationService(BaseModel):
                     "ChemIDplus": {
                         "data_license": "custom",
                         "data_license_url": "https://www.nlm.nih.gov/databases/download/terms_and_conditions.html",  # noqa: E501
-                        "version": "20200327",
-                        "data_url": "ftp://ftp.nlm.nih.gov/nlmdata/.chemidlease/",  # noqa: E501
+                        "version": "20200327", "data_url": "ftp://ftp.nlm.nih.gov/nlmdata/.chemidlease/",  # noqa: E501
                         "rdp_url": None,
                         "data_license_attributes": {
                             "non_commercial": False,
@@ -532,5 +498,201 @@ class NormalizationService(BaseModel):
                             "share_alike": False
                         }
                     }
+                },
+                "service_meta_": {
+                    'name': 'thera-py',
+                    'version': '0.1.0',
+                    'response_datetime': '2021-04-05T16:44:15.367831',
+                    'url': 'https://github.com/cancervariants/therapy-normalization'  # noqa: E501
+                }
+            }
+
+
+class SearchService(BaseModel):
+    """Core response schema containing matches for each source"""
+
+    query: str
+    warnings: Optional[Dict]
+    source_matches: Union[Dict[SourceName, MatchesKeyed], List[MatchesListed]]
+    service_meta_: ServiceMeta
+
+    class Config:
+        """Enables orm_mode"""
+
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any],
+                         model: Type['SearchService']) -> None:
+            """Configure OpenAPI schema"""
+            if 'title' in schema.keys():
+                schema.pop('title', None)
+            for prop in schema.get('properties', {}).values():
+                prop.pop('title', None)
+            schema['example'] = {
+                "query": "cisplatin",
+                "warnings": None,
+                "source_matches": [
+                    {
+                        "source": "ChemIDplus",
+                        "match_type": 80,
+                        "records": [
+                            {
+                                "label": "Cisplatin",
+                                "concept_id": "chemidplus:15663-27-1",
+                                "aliases": [
+                                    "cis-Diaminedichloroplatinum",
+                                    "1,2-Diaminocyclohexaneplatinum II citrate"
+                                ],
+                                "other_identifiers": ["drugbank:DB00515"],
+                                "xrefs": ["fda:Q20Q21Q62J"],
+                                "approval_status": None,
+                                "trade_names": []
+                            }
+                        ],
+                        "source_meta_": {
+                            "data_license": "custom",
+                            "data_license_url": "https://www.nlm.nih.gov/databases/download/terms_and_conditions.html",  # noqa: E501
+                            "version": "20210204",
+                            "data_url": "ftp://ftp.nlm.nih.gov/nlmdata/.chemidlease/",  # noqa: E501
+                            "rdp_url": None,
+                            "data_license_attributes": {
+                                "non_commercial": False,
+                                "attribution": True,
+                                "share_alike": False
+                            }
+                        }
+                    },
+                    {
+                        "source": "RxNorm",
+                        "match_type": 80,
+                        "records": [
+                            {
+                                "label": "cisplatin",
+                                "concept_id": "rxcui:2555",
+                                "aliases": [
+                                    "cis-Dichlorodiammineplatinum(II)",
+                                    "Platinum Diamminodichloride",
+                                    "cis Diamminedichloroplatinum",
+                                    "cis-diamminedichloroplatinum(II)",
+                                    "cis-Diamminedichloroplatinum",
+                                    "cis Platinum",
+                                    "CDDP",
+                                    "Dichlorodiammineplatinum",
+                                    "cis-Platinum",
+                                    "CISplatin",
+                                    "cis-Diamminedichloroplatinum(II)",
+                                    "Cis-DDP",
+                                    "DDP",
+                                    "Diamminodichloride, Platinum"
+                                ],
+                                "other_identifiers": [
+                                    "drugbank:DB00515",
+                                    "drugbank:DB12117"
+                                ],
+                                "xrefs": [
+                                    "usp:m17910",
+                                    "vandf:4018139",
+                                    "mesh:D002945",
+                                    "mthspl:Q20Q21Q62J",
+                                    "mmsl:d00195",
+                                    "atc:L01XA01",
+                                    "mmsl:31747",
+                                    "mmsl:4456"
+                                ],
+                                "approval_status": "approved",
+                                "trade_names": [
+                                    "Cisplatin",
+                                    "Platinol"
+                                ]
+                            }
+                        ],
+                        "source_meta_": {
+                            "data_license": "UMLS Metathesaurus",
+                            "data_license_url": "https://www.nlm.nih.gov/research/umls/rxnorm/docs/termsofservice.html",  # noqa: E501
+                            "version": "20210104",
+                            "data_url": "https://www.nlm.nih.gov/research/umls/rxnorm/docs/rxnormfiles.html",  # noqa: E501
+                            "rdp_url": None,
+                            "data_license_attributes": {
+                                "non_commercial": False,
+                                "attribution": True,
+                                "share_alike": False
+                            }
+                        }
+                    },
+                    {
+                        "source": "NCIt",
+                        "match_type": 80,
+                        "records": [
+                            {
+                                "label": "Cisplatin",
+                                "concept_id": "ncit:C376",
+                                "aliases": [],
+                                "other_identifiers": ["chemidplus:15663-27-1"],
+                                "xrefs": [
+                                    "umls:C0008838",
+                                    "fda:Q20Q21Q62J",
+                                    "chebi:CHEBI:27899"
+                                ],
+                                "approval_status": None,
+                                "trade_names": []
+                            }
+                        ],
+                        "source_meta_": {
+                            "data_license": "CC BY 4.0",
+                            "data_license_url": "https://creativecommons.org/licenses/by/4.0/legalcode",  # noqa: E501
+                            "version": "20.09d",
+                            "data_url": "https://evs.nci.nih.gov/ftp1/NCI_Thesaurus/archive/2020/20.09d_Release/",  # noqa: E501
+                            "rdp_url": "http://reusabledata.org/ncit.html",
+                            "data_license_attributes": {
+                                "non_commercial": False,
+                                "attribution": True,
+                                "share_alike": False
+                            }
+                        }
+                    },
+                    {
+                        "source": "Wikidata",
+                        "match_type": 80,
+                        "records": [
+                            {
+                                "label": "cisplatin",
+                                "concept_id": "wikidata:Q412415",
+                                "aliases": [
+                                    "Platinol",
+                                    "cis-diamminedichloroplatinum(II)",
+                                    "CDDP",
+                                    "Cis-DDP",
+                                    "CIS-DDP",
+                                    "Platinol-AQ"
+                                ],
+                                "other_identifiers": [
+                                    "chemidplus:15663-27-1",
+                                    "chembl:CHEMBL11359",
+                                    "rxcui:2555",
+                                    "drugbank:DB00515"
+                                ],
+                                "xrefs": ["pubchem.compound:5702198"],
+                                "approval_status": None,
+                                "trade_names": []
+                            }
+                        ],
+                        "source_meta_": {
+                            "data_license": "CC0 1.0",
+                            "data_license_url": "https://creativecommons.org/publicdomain/zero/1.0/",  # noqa: E501
+                            "version": "20210331",
+                            "data_url": None,
+                            "rdp_url": None,
+                            "data_license_attributes": {
+                                "non_commercial": False,
+                                "attribution": False,
+                                "share_alike": False
+                            }
+                        }
+                    }
+                ],
+                "service_meta_": {
+                    'name': 'thera-py',
+                    'version': '0.1.0',
+                    'response_datetime': '2021-04-05T16:44:15.367831',
+                    'url': 'https://github.com/cancervariants/therapy-normalization'  # noqa: E501
                 }
             }
